@@ -4,7 +4,7 @@
 
 ;; Author: Jorgen Schaefer <contact@jorgenschaefer.de>
 ;; URL: http://github.com/jorgenschaefer/pyvenv
-;; Version: 1.10
+;; Version: 1.17
 ;; Keywords: Python, Virtualenv, Tools
 
 ;; This program is free software; you can redistribute it and/or
@@ -97,6 +97,16 @@ educated guess, but that can be off."
   :safe #'file-directory-p
   :group 'pyvenv)
 
+(defcustom pyvenv-exec-shell
+  (or (executable-find "bash")
+      (executable-find "sh")
+      shell-file-name)
+  "The path to a POSIX compliant shell to use for running
+  virtualenv hooks. Useful if you use a non-POSIX shell (e.g.
+  fish)."
+  :type '(file :must-match t)
+  :group 'pyvenv)
+
 ;; API for other libraries
 
 (defvar pyvenv-virtual-env nil
@@ -110,6 +120,15 @@ Do not set this variable directly; use `pyvenv-activate' or
 
 This is usually the base name of `pyvenv-virtual-env'.")
 
+
+(defvar pyvenv-pre-create-hooks nil
+  "Hooks run before a virtual environment is created.")
+
+
+(defvar pyvenv-post-create-hooks nil
+  "Hooks run after a virtual environment is created.")
+
+
 (defvar pyvenv-pre-activate-hooks nil
   "Hooks run before a virtual environment is activated.
 
@@ -145,6 +164,26 @@ This is usually the base name of `pyvenv-virtual-env'.")
 (defvar pyvenv-old-eshell-path nil
   "The old eshell path before the last activate.")
 
+
+(defun pyvenv-create (venv-name python-executable)
+  "Create virtualenv.  VENV-NAME  PYTHON-EXECUTABLE."
+  (interactive (list
+                (read-from-minibuffer "Name of virtual environment: ")
+                (read-file-name "Python interpreter to use: "
+                                (file-name-directory (executable-find "python"))
+                                nil nil "python")))
+  (let ((venv-dir (concat (file-name-as-directory (pyvenv-workon-home))
+                          venv-name)))
+    (unless (file-exists-p venv-dir)
+      (run-hooks 'pyvenv-pre-create-hooks)
+      (with-current-buffer (generate-new-buffer "*virtualenv*")
+        (call-process "virtualenv" nil t t
+                      "-p" python-executable venv-dir)
+        (display-buffer (current-buffer)))
+      (run-hooks 'pyvenv-post-create-hooks))
+    (pyvenv-activate venv-dir)))
+
+
 ;;;###autoload
 (defun pyvenv-activate (directory)
   "Activate the virtual environment in DIRECTORY."
@@ -156,6 +195,13 @@ This is usually the base name of `pyvenv-virtual-env'.")
                                  (directory-file-name directory))
         python-shell-virtualenv-path directory
         python-shell-virtualenv-root directory)
+  ;; Set venv name as parent directory for generic directories
+  (when (member pyvenv-virtual-env-name '("venv" ".venv"))
+    (setq pyvenv-virtual-env-name
+          (file-name-nondirectory
+           (directory-file-name
+            (file-name-directory
+             (directory-file-name directory))))))
   ;; Preserve variables from being overwritten.
   (let ((old-exec-path exec-path)
         (old-eshell-path eshell-path-env)
@@ -268,7 +314,11 @@ configured."
       (dolist (name (directory-files workon-home))
         (when (or (file-exists-p (format "%s/%s/bin/activate"
                                          workon-home name))
+                  (file-exists-p (format "%s/%s/bin/python"
+                                         workon-home name))
                   (file-exists-p (format "%s/%s/Scripts/activate.bat"
+                                         workon-home name))
+                  (file-exists-p (format "%s/%s/python.exe"
                                          workon-home name)))
           (setq result (cons name result))))
       (sort result (lambda (a b)
@@ -381,7 +431,8 @@ CAREFUL! This will modify your `process-environment' and
 `exec-path'."
   (when (pyvenv-virtualenvwrapper-supported)
     (with-temp-buffer
-      (let ((tmpfile (make-temp-file "pyvenv-virtualenvwrapper-")))
+      (let ((tmpfile (make-temp-file "pyvenv-virtualenvwrapper-"))
+            (shell-file-name pyvenv-exec-shell))
         (unwind-protect
             (let ((default-directory (pyvenv-workon-home)))
               (apply #'call-process
